package device.demo.serial;

import android.app.AlertDialog;
import android.app.ProgressDialog;
import android.content.DialogInterface;
import android.graphics.Color;
import android.os.Bundle;
import android.os.Handler;
import android.os.RemoteException;
import android.os.ServiceManager;
import android.util.Log;
import android.widget.LinearLayout;
import android.widget.ScrollView;
import android.widget.TextView;
import device.service.DeviceService;
import device.service.IDeviceService;

public class OEM615Activity extends BaseActivity {
	private static final String TAG = OEM615Activity.class.getSimpleName();

	private IDeviceService mDS;
	private ProgressDialog mProgress;
	private Handler mHandler = new Handler();
	private RunnableCheckBaud mRunnableCheckBaud = new RunnableCheckBaud();

	private String[] mBaudrateItems;
	private String mBaudrateItem;
	private static final int MAX_LINES = 200;
	private static ScrollView mScroll;
	private static LinearLayout mLayout;
	private static boolean mInitSuccess = false;
	private static Runnable mRunnableScroll = new Runnable() {
		@Override
		public void run() {
			mScroll.fullScroll(ScrollView.FOCUS_DOWN);
		}
	};

	private Runnable mRunnableNmea = new Runnable() {
		@Override
		public void run() {
			if (mProgress.isShowing()) {
				mProgress.dismiss();
			}
			String[] commandList = getResources().getStringArray(R.array.enabled_oem615_nmea);
			for (String command : commandList) {
				writeData(command);
				writeData('\n');
			}
		}
	};

	private Runnable mRunnableChangeSerialPort = new Runnable() {
		@Override
		public void run() {
			changeSerialPort(Integer.parseInt(mBaudrateItem));
			writeData("saveconfig");
			writeData('\n');
			mHandler.postDelayed(mRunnableNmea, 3000);
		}
	};

	private class RunnableCheckBaud implements Runnable {
		private int mRetry = 0;
		private boolean isChangedBaud = false;

		/* find current baud rate 9600 or 115200 */
		@Override
		public void run() {
			if (mInitSuccess) {
				if (mProgress.isShowing()) {
					mProgress.dismiss();
				}
				showSetBaudrate();
			} else if (mRetry == 0) {
				mRetry++;
				/* get com config */
				writeData("log comconfig once");
				writeData('\n');
				writeData("log portstats once");
				writeData('\n');
				mHandler.postDelayed(mRunnableCheckBaud, 500);
			} else if (mRetry > 10) {
				if (isChangedBaud) {
					if (mProgress.isShowing()) {
						mProgress.dismiss();
					}
					writeTextView("Please re-try");
				} else {
					mRetry = 0;
					isChangedBaud = true;
					changeSerialPort(115200);
					mHandler.postDelayed(mRunnableCheckBaud, 1000);
				}
			} else {
				mRetry++;
				mHandler.postDelayed(mRunnableCheckBaud, 500);
			}
		}
	};

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		Log.d(TAG, "onCreate()");
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_oem615);

		mDS = IDeviceService.Stub.asInterface(ServiceManager.getService(DeviceService.TAG));
		try {
			mDS.setGpsPower(true);
		} catch (RemoteException e) {
			e.printStackTrace();
		}

		mBaudrateItems = OEM615Activity.this.getResources().getStringArray(R.array.oem615_baudrates);
		mBaudrateItem = mBaudrateItems[0];

		setToReadLine();
		((PortApplication) getApplication()).setProfile("/dev/ttyO0", Integer.parseInt(mBaudrateItem));

		mScroll = (ScrollView) findViewById(R.id.scrollReception);
		mLayout = (LinearLayout) findViewById(R.id.layoutReception);

		mProgress = ProgressDialog.show(OEM615Activity.this, "", "Loading, Please wait...");
		mHandler.postDelayed(mRunnableCheckBaud, 10000);
	}

	private void changeSerialPort(int baud) {
		closeSerialPort();
		((PortApplication) getApplication()).setProfile("/dev/ttyO0", baud);
		initSerialPort();
	}

	private void showSetBaudrate() {
		AlertDialog.Builder alert = new AlertDialog.Builder(OEM615Activity.this);
		alert.setTitle("Select baudrate for OEM615");

		alert.setSingleChoiceItems(mBaudrateItems, 0, new DialogInterface.OnClickListener() {
			public void onClick(DialogInterface dialog, int whichButton) {
				mBaudrateItem = mBaudrateItems[whichButton];
			}
		});

		alert.setPositiveButton(android.R.string.ok, new DialogInterface.OnClickListener() {
			public void onClick(DialogInterface dialog, int whichButton) {
				mProgress = ProgressDialog.show(OEM615Activity.this, "", "Loading, Please wait...");
				/* config serial baud rate */
				writeData("serialconfig com1 " + mBaudrateItem + " n 8 1 n");
				writeData('\n');
				mHandler.postDelayed(mRunnableChangeSerialPort, 5000);
			}
		});

		alert.setNegativeButton(android.R.string.cancel, new DialogInterface.OnClickListener() {
			public void onClick(DialogInterface dialog, int whichButton) {
				finish();
			}
		});
		alert.show();
	}

	private void writeTextView(String buffer) {
		TextView tv = new TextView(getBaseContext());
		tv.setText(buffer);
		if (buffer.contains("GPGGA")) {
			tv.setTextColor(Color.RED);
		}
		mLayout.addView(tv);
		mScroll.post(mRunnableScroll);
	}

	@Override
	protected void onDestroy() {
		if (mDS != null) {
			try {
				mDS.setGpsPower(false);
			} catch (RemoteException e) {
				e.printStackTrace();
			}
		}
		mInitSuccess = false;
		super.onDestroy();
	}

	@Override
	protected void onDataReceived(final String buffer) {
		runOnUiThread(new Runnable() {
			public void run() {
				if (!mInitSuccess) {
					mInitSuccess = true;
					return;
				}

				if (mLayout.getChildCount() > MAX_LINES) {
					mLayout.removeViewAt(0);
				}

				writeTextView(buffer);
			}
		});
	}
}
